@@ -888,10 +888,10 @@ update_rectangles (GladeDesignLayoutPrivate *priv, GtkAllocation *alloc)
   priv->layout_width = width + (OUTLINE_WIDTH*2);
   width = MIN (alloc->width, width);
 
-  rect->x = alloc->x + priv->child_offset + alloc->width - width - OUTLINE_WIDTH/2;
-  rect->y = alloc->y + priv->child_offset + alloc->height + OUTLINE_WIDTH/2;
+  rect->x = alloc->x + priv->child_offset + alloc->width - width - OUTLINE_WIDTH;
+  rect->y = alloc->y + priv->child_offset + alloc->height;
   rect->width = width + (OUTLINE_WIDTH*2);
-  rect->height = height + OUTLINE_WIDTH;
+  rect->height = height + (OUTLINE_WIDTH*1.5);
 
   /* Update south rectangle width */
   priv->south.width = rect->x - priv->south.x;
@@ -948,7 +948,16 @@ update_widget_name (GladeDesignLayout *layout, GladeWidget *gwidget)
 
   if (priv->widget_name && gwidget)
     {
-      pango_layout_set_text (priv->widget_name, glade_widget_get_name (gwidget), -1);
+      if (glade_widget_has_name (gwidget))
+        {
+          pango_layout_set_text (priv->widget_name, glade_widget_get_display_name (gwidget), -1);
+        }
+      else
+        {
+          GladeWidgetAdaptor *adaptor = glade_widget_get_adaptor (gwidget);
+          pango_layout_set_text (priv->widget_name, glade_widget_adaptor_get_name (adaptor), -1);
+        }
+
       gtk_widget_queue_resize (GTK_WIDGET (layout));
     }
 }
@@ -964,10 +973,13 @@ glade_design_layout_add (GtkContainer *container, GtkWidget *widget)
 {
   GladeDesignLayout *layout = GLADE_DESIGN_LAYOUT (container);
   GladeDesignLayoutPrivate *priv = layout->priv;
+  GtkStyleContext *context = gtk_widget_get_style_context (widget);
 
   priv->child_rect.width = 0;
   priv->child_rect.height = 0;
 
+  gtk_style_context_add_class (context, "background");
+
   gtk_widget_set_parent_window (widget, priv->offscreen_window);
 
   GTK_CONTAINER_CLASS (glade_design_layout_parent_class)->add (container,
@@ -1012,67 +1024,26 @@ glade_design_layout_damage (GtkWidget *widget, GdkEventExpose *event)
 }
 
 static inline void
-draw_frame (cairo_t *cr, GladeDesignLayoutPrivate *priv, gboolean selected,
+draw_frame (GtkWidget *widget, cairo_t *cr, gboolean selected,
             int x, int y, int w, int h)
 {
-  cairo_save (cr);
+  GladeDesignLayoutPrivate *priv = ((GladeDesignLayout *)widget)->priv;
+  GtkStyleContext *context = gtk_widget_get_style_context (widget);
 
-  cairo_set_line_width (cr, OUTLINE_WIDTH);
-
-  cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
-  cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
-
-  gdk_cairo_set_source_rgba (cr, (selected) ? &priv->frame_color_active[0] :
-                             &priv->frame_color[0]);
-
-  /* rectangle */
-  cairo_rectangle (cr, x, y, w, h);
-  cairo_stroke (cr);
+  gtk_render_background (context, cr, x, y, w, h);
+  gtk_render_frame (context, cr, x, y, w, h);
 
   if (priv->widget_name)
     {
-      GdkRGBA *color = (selected) ? &priv->frame_color_active[1] : &priv->frame_color[1];
       GdkRectangle *rect = &priv->south_east;
-      cairo_pattern_t *pattern;
-      gdouble xx, yy;
-      
-      xx = rect->x + rect->width;
-      yy = rect->y + rect->height;
-
-      /* Draw tab */
-      cairo_move_to (cr, rect->x, rect->y);
-      cairo_line_to (cr, xx, rect->y);
-      cairo_line_to (cr, xx, yy-8);
-      cairo_curve_to (cr, xx, yy, xx, yy, xx-8, yy);
-      cairo_line_to (cr, rect->x+8, yy);
-      cairo_curve_to (cr, rect->x, yy, rect->x, yy, rect->x, yy-8);
-      cairo_close_path (cr);
-      cairo_fill (cr);
-
-      /* Draw widget name */
-      if (rect->width < priv->layout_width)
-        {
-          gdouble r = color->red, g = color->green, b = color->blue;
-          
-          pattern = cairo_pattern_create_linear (xx-16-OUTLINE_WIDTH, 0,
-                                                 xx-OUTLINE_WIDTH, 0);
-          cairo_pattern_add_color_stop_rgba (pattern, 0, r, g, b, 1);
-          cairo_pattern_add_color_stop_rgba (pattern, 1, r, g, b, 0);
-          cairo_set_source (cr, pattern);
-        }
-      else
-        {
-          pattern = NULL;
-          gdk_cairo_set_source_rgba (cr, color);
-        }
-
-      cairo_move_to (cr, rect->x + OUTLINE_WIDTH, rect->y + OUTLINE_WIDTH);
-      pango_cairo_show_layout (cr, priv->widget_name);
-
-      if (pattern) cairo_pattern_destroy (pattern);
+      gtk_style_context_save (context);
+      gtk_style_context_add_class (context, "handle");
+      gtk_render_background (context, cr, rect->x, rect->y, rect->width, rect->height);
+      gtk_render_frame (context, cr, rect->x, rect->y, rect->width, rect->height);
+      gtk_render_layout (context, cr, rect->x + OUTLINE_WIDTH, rect->y + OUTLINE_WIDTH,
+                         priv->widget_name);
+      gtk_style_context_restore (context);
     }
-
-  cairo_restore (cr);
 }
 
 static void
@@ -1107,14 +1078,17 @@ draw_selection (cairo_t *cr,
 {
   gint x, y, w, h, xw, yh, y_top, yh_bottom, x_left, xw_right;
   gint top, bottom, left, right;
-  cairo_pattern_t *gradient;
-  gdouble r, g, b, cx, cy;
+  gdouble r, g, b;
   GtkAllocation alloc;
+  GtkStyleContext *context;
 
   gtk_widget_get_allocation (widget, &alloc);
 
   if (alloc.x < 0 || alloc.y < 0) return;
-  
+
+  context = gtk_widget_get_style_context (parent);
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, "selection");
   r = color->red; g = color->green; b = color->blue;
   gtk_widget_translate_coordinates (widget, parent, 0, 0, &x, &y);
 
@@ -1129,19 +1103,9 @@ draw_selection (cairo_t *cr,
   yh_bottom = yh + bottom;
   x_left = x - left;
   xw_right = xw + right;
-  
+
   /* Draw widget area overlay */
-  cx = x + w/2;
-  cy = y + h/2;
-  gradient = cairo_pattern_create_radial (cx, cy, MIN (w, h)/6, cx, cy, MAX (w, h)/2);
-  cairo_pattern_add_color_stop_rgba (gradient, 0, r+.24, g+.24, b+.24, .16);
-  cairo_pattern_add_color_stop_rgba (gradient, 1, r, g, b, .28);
-  cairo_set_source (cr, gradient);
-
-  cairo_rectangle (cr, x, y, w, h);
-  cairo_fill (cr);
-
-  cairo_pattern_destroy (gradient);
+  gtk_render_background (context, cr, x, y, w, h);
 
   /* Draw margins overlays */
   if (top)
@@ -1161,9 +1125,8 @@ draw_selection (cairo_t *cr,
                            r, g, b, xw_right, y);
 
   /* Draw Selection box */
-  cairo_set_source_rgba (cr, r, g, b, .75);
-  cairo_rectangle (cr, x - left, y - top, w + left + right, h + top + bottom);
-  cairo_stroke (cr);
+  gtk_render_frame (context, cr, x - left, y - top, w + left + right, h + top + bottom);
+  gtk_style_context_restore (context);
 }
 
 #define DIMENSION_OFFSET 9
@@ -1544,6 +1507,13 @@ glade_design_layout_draw (GtkWidget *widget, cairo_t *cr)
           gboolean selected = FALSE;
           GList *l;
 
+          /* draw frame */
+          draw_frame (widget, cr, selected,
+                      border_width + PADDING,
+                      border_width + PADDING,
+                      priv->child_rect.width + 2 * OUTLINE_WIDTH,
+                      priv->child_rect.height + 2 * OUTLINE_WIDTH);
+
           /* draw offscreen widgets */
           gdk_cairo_set_source_window (cr, priv->offscreen_window,
                                        priv->child_offset, priv->child_offset);
@@ -1582,13 +1552,6 @@ glade_design_layout_draw (GtkWidget *widget, cairo_t *cr)
                 selected = TRUE;
             }
 
-          /* draw frame */
-          draw_frame (cr, priv, selected,
-                      border_width + PADDING,
-                      border_width + PADDING,
-                      priv->child_rect.width + 2 * OUTLINE_WIDTH,
-                      priv->child_rect.height + 2 * OUTLINE_WIDTH);
-
           /* Draw selection nodes if we are in margins edit mode */
           if (priv->selection && gtk_widget_is_ancestor (priv->selection, child))
             draw_selection_nodes (cr, priv, widget);
@@ -1817,8 +1780,7 @@ glade_design_layout_style_updated (GtkWidget *widget)
 {
   GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (widget);
   
-  _glade_design_layout_get_colors (gtk_widget_get_style_context (widget),
-                                   &priv->frame_color[0],
+  _glade_design_layout_get_colors (&priv->frame_color[0],
                                    &priv->frame_color[1],
                                    &priv->frame_color_active[0],
                                    &priv->frame_color_active[1]);
@@ -2046,6 +2008,7 @@ glade_design_layout_class_init (GladeDesignLayoutClass * klass)
   GObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
+  GtkCssProvider *css_provider;
 
   object_class = G_OBJECT_CLASS (klass);
   widget_class = GTK_WIDGET_CLASS (klass);
@@ -2088,6 +2051,17 @@ glade_design_layout_class_init (GladeDesignLayoutClass * klass)
                                    GLADE_TYPE_DESIGN_LAYOUT,
                                    g_cclosure_new (G_CALLBACK (glade_design_layout_damage),
                                                    NULL, NULL));
+
+  /* Setup Custom CSS */
+  gtk_widget_class_set_css_name (widget_class, "glade-design-layout");
+
+  css_provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_resource (css_provider, "/org/gnome/gladeui/glade-design-layout.css");
+
+  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                             GTK_STYLE_PROVIDER (css_provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (css_provider);
 }
 
 /* Internal API */
@@ -2205,23 +2179,37 @@ _glade_design_layout_coords_from_event (GdkWindow *parent,
 }
 
 void
-_glade_design_layout_get_colors (GtkStyleContext *context, 
-                                 GdkRGBA *c1, GdkRGBA *c2,
+_glade_design_layout_get_colors (GdkRGBA *c1, GdkRGBA *c2,
                                  GdkRGBA *c3, GdkRGBA *c4)
 {
+  GtkStyleContext *context = gtk_style_context_new ();
+  GtkWidgetPath *path = gtk_widget_path_new ();
   gfloat off;
-  
-  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, c1);
-  gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, c2);
 
-  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_SELECTED | GTK_STATE_FLAG_FOCUSED, c3);
-  gtk_style_context_get_color (context, GTK_STATE_FLAG_SELECTED | GTK_STATE_FLAG_FOCUSED, c4);
+  /* Fake style context */
+  gtk_widget_path_append_type (path, GTK_TYPE_WIDGET);
+  gtk_style_context_set_path (context, path);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+
+  /* Get colors */
+  gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+  gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), c1);
+  gtk_style_context_get_color (context, gtk_style_context_get_state (context), c2);
+
+  gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL | GTK_STATE_FLAG_SELECTED | GTK_STATE_FLAG_FOCUSED);
+  gtk_style_context_set_state (context, gtk_style_context_get_state (context));
+  gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), c3);
+  gtk_style_context_get_color (context, gtk_style_context_get_state (context), c4);
 
   off = ((c1->red + c1->green + c1->blue)/3 < .5) ? .16 : -.16;
    
   c1->red += off;
   c1->green += off;
   c1->blue += off;
+
+  /* Free resources */
+  gtk_widget_path_free (path);
+  g_object_unref (context);
 }
 
 void
